﻿using System;
using UnityEngine;

public class Archery : MonoBehaviour
{
    /// <summary>
    /// Define some states for the archer
    /// </summary>
    public enum ArcherState
    {
        Rest,
        Reload,
        Fire,
        Aim,
        None
    }

    // What prefabs do we have?
    public GameObject ArrowPrefab;

    // What state are we in?
    private ArcherState currentState = ArcherState.Rest;
    private ArcherState lastState = ArcherState.None;

    private Animator animamatorReference;

    private float firingPower = 0f;
    public int arrows = 10;
    public Transform Hand;

    // Setup our scene
    void Start()
    {
        animamatorReference = GetComponent<Animator>();
    }

    // Update is called once per frame
    void Update()
    {
        if (Input.GetMouseButtonDown(1))
            currentState = ArcherState.Aim;

        if (Input.GetMouseButtonUp(1))
            currentState = ArcherState.Fire;

        if (Input.GetKey(KeyCode.Q))
            currentState = ArcherState.Reload;

        if (currentState != lastState)
        {
            // the state has changed, time to change our animation
            AnimateArcher();
            if (currentState != lastState)
                Debug.Log(name + "::Upddate > Archer animation state changed to " + currentState.ToString());
        }

        // Execute our aiming, firing, reload, etc
        RunActions();

        // Update our reference, so we can determine if a state is "held" or if a a state is "new".
        lastState = currentState;
    }


    void RunActions()
    {
        // What state is the character currently in?
        switch (currentState)
        {
            case ArcherState.Aim:
                // Execute every time
                IncreasePower();
                break;
            case ArcherState.Reload:
                // Only execute once
                if (currentState != lastState)
                    Reload();
                // Then back to resting
                currentState = ArcherState.Rest;
                break;
            case ArcherState.Fire:
                // Only execute once
                if (currentState != lastState)
                    Fire();
                // Then back to resting
                currentState = ArcherState.Rest;
                break;
        }
    }


    void IncreasePower()
    {
        float minPower = 10f;
        float maxPower = 100f;
        firingPower = Mathf.Clamp(firingPower += 1f, minPower, maxPower); 
        Debug.Log("Power " + firingPower);
    }

    void Fire()
    {
        // Ammo?
        if (arrows == 0)
        {
            Debug.Log("Need ammo");
            return;
        }

        // Fire prefab
        var fireArrow = GameObject.Instantiate(ArrowPrefab);
        if (fireArrow != null)
        {
            fireArrow.transform.rotation = Hand.rotation;
            fireArrow.transform.position = Hand.position;

            // Get or add a rigid body for phyiscs
            var fireArrowRigid = fireArrow.GetComponent<Rigidbody>();
            if (fireArrowRigid == null)
                fireArrowRigid = fireArrow.AddComponent<Rigidbody>();

            // then apply rigidbody or whatever with a direction of transform.position.forward against the firing power;
            fireArrowRigid.useGravity = true;
            fireArrowRigid.AddForce(new Vector3(10f,15f,0f) * firingPower);

            firingPower = 0f;
        }

        // Use up ammo
        arrows -= 1;
    }

    void Reload()
    {
        // Reload 
        arrows = 10;
    }

    void AnimateArcher()
    {
        // Animate based on the characters state
    }
}
